@@ -29,28 +29,24 @@
 Rcpp::List custom_train(Rcpp::NumericMatrix input_data,
                         Eigen::SparseMatrix<double> sparse_input_data,
                         size_t outcome_index,
-                        std::vector <std::string> variable_names,
                         unsigned int mtry,
                         unsigned int num_trees,
-                        bool verbose,
                         unsigned int num_threads,
                         unsigned int min_node_size,
-                        bool sample_with_replacement,
-                        bool keep_inbag,
                         double sample_fraction,
-                        std::vector<size_t> no_split_variables,
                         unsigned int seed,
                         bool honesty,
                         unsigned int ci_group_size,
-                        double alpha) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data, variable_names);
+                        double alpha,
+                        double imbalance_penalty,
+                        std::vector<size_t> clusters,
+                        unsigned int samples_per_cluster) {
+  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
+  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size,
+      honesty, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
 
-  ForestTrainer trainer = ForestTrainers::custom_trainer(data,
-          outcome_index - 1,
-          alpha);
-  RcppUtilities::initialize_trainer(trainer, mtry, num_trees, num_threads, min_node_size,
-                                    sample_with_replacement, sample_fraction, no_split_variables, seed, honesty, ci_group_size);
-  Forest forest = trainer.train(data);
+  ForestTrainer trainer = ForestTrainers::custom_trainer(outcome_index - 1);
+  Forest forest = trainer.train(data, options);
 
   Rcpp::List result = RcppUtilities::create_forest_object(forest, data);
   delete data;
@@ -61,9 +57,8 @@ Rcpp::List custom_train(Rcpp::NumericMatrix input_data,
 Rcpp::NumericMatrix custom_predict(Rcpp::List forest_object,
                                    Rcpp::NumericMatrix input_data,
                                    Eigen::SparseMatrix<double> sparse_input_data,
-                                   std::vector<std::string> variable_names,
                                    unsigned int num_threads) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data, variable_names);
+  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
   Forest forest = RcppUtilities::deserialize_forest(
       forest_object[RcppUtilities::SERIALIZED_FOREST_KEY]);
 
@@ -79,9 +74,8 @@ Rcpp::NumericMatrix custom_predict(Rcpp::List forest_object,
 Rcpp::NumericMatrix custom_predict_oob(Rcpp::List forest_object,
                                        Rcpp::NumericMatrix input_data,
                                        Eigen::SparseMatrix<double> sparse_input_data,
-                                       std::vector<std::string> variable_names,
                                        unsigned int num_threads) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data, variable_names);
+  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
   Forest forest = RcppUtilities::deserialize_forest(
       forest_object[RcppUtilities::SERIALIZED_FOREST_KEY]);
 
