@@ -30,6 +30,7 @@
 #include "utility.h"
 #include "globals.h"
 #include "DefaultData.h"
+#include "SparseData.h"
 
 void split_sequence(std::vector<uint>& result, uint start, uint end, uint num_parts) {
 
@@ -161,3 +162,15 @@ Data* load_data(std::string file_name) {
   data->sort();
   return data;
 }
+
+Data* load_sparse_data(std::string file_name) {
+  Data* data = new SparseData();
+
+  bool rounding_error = data->load_from_file(file_name);
+  if (rounding_error) {
+    throw std::runtime_error("A rounding error occurred while loading data from file.");
+  }
+
+  data->sort();
+  return data;
+}
