@@ -18,8 +18,37 @@
 #include "prediction/Prediction.h"
 
 Prediction::Prediction(const std::vector<double>& predictions):
-  predictions(predictions), variance_estimates(0) {}
+  predictions(predictions),
+  variance_estimates(0),
+  error_estimates(0) {}
 
 Prediction::Prediction(const std::vector<double>& predictions,
-                       const std::vector<double>& variance_estimates):
-  predictions(predictions), variance_estimates(variance_estimates) {}
+                       const std::vector<double>& variance_estimates,
+                       const std::vector<double>& error_estimates):
+  predictions(predictions),
+  variance_estimates(variance_estimates),
+  error_estimates(error_estimates) {}
+
+const std::vector<double>& Prediction::get_predictions() const {
+  return predictions;
+}
+
+const std::vector<double>& Prediction::get_variance_estimates() const {
+  return variance_estimates;
+}
+
+const std::vector<double>& Prediction::get_error_estimates() const {
+  return error_estimates;
+}
+
+const bool Prediction::contains_variance_estimates() const {
+  return !variance_estimates.empty();
+}
+
+const bool Prediction::contains_error_estimates() const {
+  return !error_estimates.empty();
+}
+
+const size_t Prediction::size() const {
+  return predictions.size();
+}
